@@ -33,7 +33,7 @@
 #include <linux/module.h>
 #include <linux/init.h>
 #include <linux/proc_fs.h>
-#include <linux/version.h>
+#include <linux/devfs_fs_kernel.h>
 #include <asm/io.h>
 #include <asm/uaccess.h>
 #include "thinkpad_common.h"
@@ -48,20 +48,48 @@
 
 extern struct proc_dir_entry *thinkpad_ppde;
 
+#ifdef CONFIG_DEVFS_FS
+static int smapi_open(
+	struct inode * pinodeThe,
+	struct file * pfileThe
+);
+static int smapi_release(
+	struct inode * pinodeThe,
+	struct file * pfileThe 
+);
+static int smapi_ioctl(
+	struct inode * pinodeThe,
+	struct file * pfileThe,
+	unsigned int uintIoctlNum,
+	unsigned long ulongIoctlArg
+);
+#endif
 
 /****** variables ******/
 
 static const char _szMyName[] = "smapi";
 static const char _szImName[] = "smapi_do";
-static const char _szMyVersion[] = "5.0";
+static const char _szMyVersion[] = "4.0";
 static const char _szProcfile[] = "driver/thinkpad/smapi";
 
 #ifdef MODULE
 MODULE_AUTHOR( "Thomas Hood" );
 MODULE_DESCRIPTION( "Driver for the SMAPI BIOS on IBM ThinkPads" );
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,4,10)
 MODULE_LICENSE( "GPL" );
 #endif
+#endif
 
+#ifdef CONFIG_DEVFS_FS
+static struct file_operations _fileopsSmapi = {
+	.ioctl   = smapi_ioctl,
+	.open    = smapi_open,
+	.release = smapi_release,
+	.owner   = THIS_MODULE
+};
+
+static devfs_handle_t _devfs_handleSmapi;
+#endif
 
 /****** functions *******/
 
@@ -205,6 +233,62 @@ int smapi_do(
 }
 
 
+#ifdef CONFIG_DEVFS_FS
+static int smapi_open(
+	struct inode * pinodeThe,
+	struct file * pfileThe
+) {
+
+	return 0;
+}
+
+
+static int smapi_release(
+	struct inode * pinodeThe,
+	struct file * pfileThe
+) {
+
+	return 0;
+}
+
+
+static flag_t caller_has_w( struct file * pfileThe )
+{
+	return ((pfileThe->f_mode) & FMODE_WRITE) ? 1 : 0;
+}
+
+
+static int smapi_ioctl(
+	struct inode * pinodeThe,
+	struct file * pfileThe,
+	unsigned int uintIoctlNum,
+	unsigned long ulongIoctlArg
+) {
+
+#ifdef DEBUG_VERBOSE
+	printk( "%s: Doing ioctl number 0x%x\n", _szMyName, uintIoctlNum );
+#endif
+
+	switch ( uintIoctlNum ) {
+		/* should  return  at the end of each case block */
+
+		case IOCTL_SMAPI_REQUEST: 
+			return smapi_do(
+				ulongIoctlArg, 
+				caller_has_w( pfileThe )
+			);
+
+		default:
+			/* ioctl number not recognized -- do nothing */
+			return -ENOTTY;
+
+	} /* switch */
+			
+	return -ETHINKPAD_PROGRAMMING; /* We should never arrive here */
+}
+#endif /* defined(CONFIG_DEVFS_FS) */
+
+
 /*
  * Returns a pointer to the first SMAPI BIOS header found
  * with correct checksum
@@ -298,6 +382,23 @@ static int __init smapi_init( void )
 	}
 	/* proc entry created */
 
+#ifdef CONFIG_DEVFS_FS
+	_devfs_handleSmapi = devfs_register(
+		NULL /* dir */,
+		"thinkpad/smapi",
+		DEVFS_FL_DEFAULT | DEVFS_FL_AUTO_DEVNUM,
+		0 /* major */, 0 /* minor */,
+		S_IFCHR | S_IRUSR | S_IWUSR | S_IRGRP | S_IWGRP | S_IROTH,
+		&_fileopsSmapi,
+		NULL /* info */
+	);
+	if ( _devfs_handleSmapi == NULL ) {
+		printk(KERN_ERR "%s: Could not devfs_register(). Aborting.\n", _szMyName );
+		return -EIO;
+	}
+	/* devfs entry created */
+#endif
+
 	inter_module_register( _szImName, THIS_MODULE, &smapi_do );
 
 	return 0;
@@ -309,6 +410,10 @@ static void __exit smapi_exit( void )
 
 	inter_module_unregister( _szImName );
 
+#ifdef CONFIG_DEVFS_FS
+	devfs_unregister( _devfs_handleSmapi );
+#endif
+
 	remove_proc_entry( _szProcfile, NULL );
 
 	return;
